 /******************************************************************************
  *       __   _   _____   _   _        _____   _____   _____                  *
  *      |  \ | | | ____| | | | |      /  ___| /  ___| /  ___/                 *
  *      |   \| | | |__   | | | |      | |     | |     | |___                  *
  *      | |\   | |  __|  | | | |      | |     | |     \___  \                 *
  *      | | \  | | |___  | |_| |      | |___  | |___   ___| |                 *
  *      |_|  \_| |_____| \_____/      \_____| \_____| /_____/                 *
  *                                                                            *
  *                   RF & Serial Comm. Example                                *
  *                           Transmitter                                      *
  ******************************************************************************
  * This program demonstrates the use of the CC2420DB library, including the   *
  * basic RF library and serial link library. The packet protocol being used is*
  * a small subset of the IEEE 802.15.4 standard. It uses an 802.15.4 MAC      *
  * compatible frame format, but does not implement any other MAC functions /  *
  * mechanisms (e.g. CSMA-CA). The basic RF library can thus not be used to    *
  * communicate with compliant 802.15.4 networks.                              *
  *                                                                            *
  * A MSP430F1611 running this program will establish a point-to-point RF link *
  * on channel 26 with a CC2420DB using the following node addresses:          *
  * - PAN ID: 0x2420 (both nodes)                                              *
  * - Short local address: 0x5678                                              *
  * - Destination address: 0x1234                                              *
  *                                                                            *
  * The transmitter mote accepts user's input by either press the user-switch  *
  * button on the mote board or a SPACE from the serial link to toggle the blue*
  * LED and transmit the LED value to the destination mote which will change   *
  * the yellow LED corresponsively.                                            *
  * Please note that there is no so-called (PAN) coordinator.                  *
  *                                                                            *
  * INSTRUCTIONS:                                                              *
  * Outgoing data packets containing a 10-byte payload will be transmitted if a*
  * user input happens. Byte 0 contains the LED value, byte 1 contains the     *
  * status byte from CC2420.                                                   *
  * After downloading the program to the device, you may need to press the     *
  * RESET button if it doesn't work.                                           *
  * This example is based on the blinking LED example for the CC2420DB board.  *
  ******************************************************************************
  * Compiler: msp430-gcc GNU C compiler                                        *
  * Target platform: MSP-430F1611 (can easily be ported to other platforms)    *
  ******************************************************************************
  * Last update: 01/16/2006                                                    *
  ******************************************************************************/

#include "include/include.h"
#include "include/hardware.h"
//swuart.h is for the software serial communication.
#include "include/swuart.h"
#include <stdlib.h>
#include <stdio.h>

//Defintions used locally in this file.
#define PAYLOAD_SIZE	10
#define RF_CHANNEL	26
#define FILL_BYTE    0xEE

//Basic RF transmission structures.
BASIC_RF_TX_INFO rfTxInfo;
BASIC_RF_RX_INFO rfRxInfo;
BYTE pTxBuffer[BASIC_RF_MAX_PAYLOAD_SIZE];
BYTE pRxBuffer[BASIC_RF_MAX_PAYLOAD_SIZE];

/*
 *  void user_switch_init(void)
 *  
 *  DESCRIPTION:
 *    Initialize the interrupt for user switch that is P2.7
 */
void user_switch_init(void){
	//Select the interrupt edge of user switch to be low-to-high transition .
	P2IES &=~0x80;
	//Enable the user switch interrupt.
	P2IE |= 0x80;
} //user_switch_init.

/*
 *  void run(void)
 *
 *  DESCRIPTION:
 *    Basic logic of this example, it toggles the blue LED and sends
 *    the LED value through the RF.
 */
void run(void){
	UINT8 status;
	FASTSPI_UPD_STATUS(status);
	pTxBuffer[0]= (P5OUT&BM(BLED))?1:0;
	pTxBuffer[1]=  status;

	printf("Sending a packet.\r\n");
		
	if (basicRfSendPacket(&rfTxInfo))
		TOGGLE_BLED();
} //run

/*
 *  int main (void)
 *
 *  DESCRIPTION:
 *    Startup routine and main loop
 */
int main (void) {
	//Initalize ports for communication with CC2420 and other peripheral units.
	PORT_INIT();
	SPI_INIT();
	SWUART_INIT();

	user_switch_init();

	//Initialize for basic RF operation.
	basicRfInit(&rfRxInfo, RF_CHANNEL, 0x2420, 0x5678);
	rfTxInfo.destAddr = 0x1234;
	
	//Initialize common protocol parameters.
	rfTxInfo.length = PAYLOAD_SIZE;
	rfTxInfo.ackRequest = TRUE;
	rfTxInfo.pPayload = pTxBuffer;
  
	//Initialize the payload buffer to the default value.
	UINT8 n;
	for (n = 0; n < PAYLOAD_SIZE; n++)
		pTxBuffer[n] = FILL_BYTE;

	//In order to get the ACK, set RF receive to on also.
	basicRfReceiveOn();	

	//The main loop:
	while (1){
		//Go to low power mode 0.
		LPM0;
		//If any character is received from the serial link it will exit the low 
		//power mode and execute the following lines.
		//To test if a SPACE char is input by the user.
		if(rxdata==32)
			run();
	}
} //main


/*
 *  BASIC_RF_RX_INFO* basicRfProcessPacket(BASIC_RF_RX_INFO *pRRI)
 *
 *  DESCRIPTION:
 *    This function is a part of the basic RF library, but must be declared by 
 *    the application. Once the application has turned on the receiver, using 
 *    basicRfReceiveOn(), all incoming packets will be received by the FIFOP 
 *    interrupt service routine. When finished, the ISR will call the 
 *    basicRfProcessPacket() function. Please note that this function must 
 *    return quickly, since the next received packet will overwrite the active
 *    BASIC_RF_RX_INFO structure (pointed to by pRRI).
 *  Notes:
 *    Although the current program is only for the transmitting, the RF receive
 *    capability is still necessary if ACK is used in the transmission. But if 
 *    the packet received is an ACK, this function won't be invoked.
 *  ARGUMENTS:
 *    BASIC_RF_RX_INFO *pRRI
 *      The reception structure, which contains all relevant info about the 
 *      received packet.
 *
 *  RETURN VALUE:
 *    BASIC_RF_RX_INFO*
 *      The pointer to the next BASIC_RF_RX_INFO structure to be used by the 
 *      FIFOP ISR. If there is only one buffer, then return pRRI.
 */
BASIC_RF_RX_INFO* basicRfProcessPacket(BASIC_RF_RX_INFO *pRRI) {
    //Process the received data.
    return pRRI;
} //basicRfReceivePacket

/*
 *  The I/O interrupt service routine (ISR)
 *
 *  DESCRIPTION:
 *    It is for the event that the user-switch button is pressed and released. 
 *    User-switch is attached to P2.7
 */
interrupt (PORT2_VECTOR) user_switch(void){
    //To test whether the interrupt is from the desired source - P2.7.
    if(P2IFG&0x80){		
    //Clear the corresponding interrupt flag.
    P2IFG&=~0x80;
    run();
		}
} //user_switch

/*
 *  CC2420 FIFOP interrupt service routine (ISR)
 *
 *  DESCRIPTION:
 *    When a packet has been completely received, this ISR will extract the data
 *    from the RX FIFO, put it into the active BASIC_RF_RX_INFO structure, and 
 *    call basicRfProcessPacket() (defined by the application). FIFO overflow 
 *    and illegally formatted packets are handled by this routine.
 *
 *  Note1: Packets are acknowledged automatically by CC2420 through the auto- 
 *    acknowledgment feature.
 *  Note2: ACK will also be received in thie ISR but basicRfProcessPacket() 
 *    won't get invoked.
 */
interrupt (PORT1_VECTOR) fifo_rx(void){
    //Clearing FIFOP Interrupt.
    CLEAR_FIFOP_INT();
		
    basicRfReceivePacket();
} //fifo_rx
